#! /usr/bin/env python3

from flask import Flask, render_template, request, redirect, url_for
from datetime import datetime
import mariadb_conn as dbc
import sys
import yaml
import os

__author__ = "Diarmuid O'Briain"
__copyright__ = "Copyright 2024, Technological University of the Shannon"
__licence__ = "European Union Public Licence v1.2"
__version__ = "2.2"
__role__ = "Main TUS Database management app"
__program__ = sys.argv[0][2:] if (sys.argv[0][:2] == "./") else sys.argv[0]

title = "TUS Database"
conf = dict()
dir_ = os.path.dirname(os.path.realpath(__file__))
tmp_ = "/tmp/"
table = str()
print(f"dbc loc: {dbc.__file__}", file=sys.stderr)

# // Get the configuration information from YAML //
with open(f"{dir_}/conf.yml", "r") as fh:
    try:
        conf = yaml.safe_load(fh)
        table = conf.pop("table")     
    except yaml.YAMLError as err:
        print(f"Error: {err}", file=sys.stderr)
        sys.exit(1)
        

def dbconnect(c):
    """Connect to the Database, create connection object and cursor"""

    try:
        conn, cur = dbc.db_connect(**conf)
        return (conn, cur)

    except:
        return redirect(url_for("login"))

def dbgetpass():
    """Get stored password"""
    with open(f"{tmp_}.pass", "r") as fh:
        return(fh.read())


# // Flask app //
app = Flask(__name__)


@app.route("/")
def index():
    """The main index page"""
    return render_template("index.html", title=title)


@app.route("/login")
def login():
    """Page to attract login password from the user"""
    msg = "Enter password for the database."
    return render_template("login.html", title=title, msg=msg)


@app.route("/home", methods=["GET", "POST"])
def home():
    """General page to give message after action"""
    print(f"Post action, home, page.", file=sys.stderr)
             
    post_data = dict(request.form)
    print(f"post_data selection: {post_data['selection']}", file=sys.stderr)     
    
    if post_data["selection"] == "login":
        """ Deal with login data """
        post_data.pop("selection")
        try:
            with open(f"{tmp_}.pass", "w") as fh:
                fh.write(f"{post_data['dbpass']}")
            return redirect(url_for("read"))
        except:
            print(f"Cannot open password file {tmp_}.pass", file=sys.stderr)
            return redirect(url_for("login"))
        
    elif post_data["selection"] == "write":
        """Deal with write data"""
        post_data.pop("selection")
        print(f"Deal with write data", file=sys.stderr)
        values = tuple(post_data.values())
        query = (table, values)
        print(f"SQL Query: {query}", file=sys.stderr)
        try:
            conf['password'] = dbgetpass()
            conn, cur = dbconnect(conf)
            response = dbc.db_insert(conn, cur, query)
            cur.close(); conn.close()
            msg = "Data entry complete."
            return redirect(url_for("read"))
        except:
            print(f"home() write: Incorrect password {conf}", file=sys.stderr)
            return redirect(url_for("login"))

    elif post_data["selection"] == "delete":
        """Deal with delete data"""
        post_data.pop("selection")
        student_no = post_data["studentno"]
        query = (table, ("Student_no", student_no))
        print(f"SQL Query: {query}", file=sys.stderr)
        try:
            conf['password'] = dbgetpass()
            conn, cur = dbconnect(conf)
            data = dbc.db_delete(conn, cur, query)
            cur.close(); conn.close()
            msg = f"User {student_no} deleted, what do you want to do next, select from the toolbar."
            return redirect(url_for("read"))
        except:
            print(f"home() delete: Incorrect password {conf}", file=sys.stderr)
            return redirect(url_for("login"))
    else:
        print(f"home() else:", file=sys.stderr)
        return redirect(url_for("login"))
        
@app.route("/read")
def read():
    """Read from the database"""
    print(f"Read from the database", file=sys.stderr)
    try:
        conf["password"] = dbgetpass()
        conn, cur = dbconnect(conf)
        data = dbc.db_select(cur, table)
        cur.close(); conn.close()
        head = ("Student no", "Username", "First name", "Last name", "e-Mail", "Role")
        return render_template("read.html", title=title, head=head, data=data)
    except:
        print(f"read(): Incorrect password {conf}", file=sys.stderr)
        return redirect(url_for("login"))                 


@app.route("/write")
def write():
    """Write to the database"""
    print(f"Write to the database", file=sys.stderr)
    try:
        conf['password'] = dbgetpass()
        conn, cur = dbconnect(conf)
        data = dbc.db_select(cur, table)
        cur.close(); conn.close()
        if len(data) == 1:
            next_no = 0
        else:
            student_no_list = sorted([x[0] for x in data if isinstance(x[0], int)])
            next_no = student_no_list[-1] + 1
        msg = "Add new entry to the database."
        print(f"write(): Rendering write.html", file=sys.stderr)
        return render_template("write.html", title=title, msg=msg, next_no=next_no)
    except:
        print(f"write(): Incorrect password {conf}", file=sys.stderr)
        return redirect(url_for("login"))            


@app.route("/delete")
def delete():
    """Delete a user from the database"""
    print(f"Delete a user from the database", file=sys.stderr)
    msg = "Delete a user from the database."
    return render_template("delete.html", title=title, msg=msg)


@app.route("/about")
def about():
    """General about page"""
    return render_template("about.html", title=title)


@app.route("/logout")
def logout():
    """Logout page"""
    try:
        with open(f"{tmp_}.pass", "w") as fh:
            fh.write(f" ")
        return redirect(url_for("login"))
    except:
        print(f"Cannot open password file .pass", file=sys.stderr)
        return redirect(url_for("login"))


if __name__ == "__main__":
    app.run(debug=True)
